import { TypedEventEmitter, Libp2p } from "@libp2p/interface";
import { Logger } from "@libp2p/logger";
import { Node, Tree } from "@canvas-js/okra";
import type { Signature, Signer, Message, Awaitable } from "@canvas-js/interfaces";
import type { AbstractModelDB, ModelSchema } from "@canvas-js/modeldb";
import type { NetworkConfig, ServiceMap } from "@canvas-js/gossiplog/libp2p";
import type { SyncSnapshot } from "./interface.js";
import { MessageSource, SignedMessage } from "./SignedMessage.js";
export type GossipLogConsumer<Payload = unknown, Result = any> = (this: AbstractGossipLog<Payload, Result>, signedMessage: SignedMessage<Payload, Result>) => Awaitable<Result>;
export interface GossipLogInit<Payload = unknown, Result = any> {
    topic: string;
    apply: GossipLogConsumer<Payload, Result>;
    signer?: Signer<Payload>;
    /** validate that the IPLD `payload` is a `Payload` type */
    validatePayload?: (payload: unknown) => payload is Payload;
    verifySignature?: (signature: Signature, message: Message<Payload>) => Awaitable<void>;
    /** add extra tables to the local database for private use */
    schema?: ModelSchema;
}
export type GossipLogEvents<Payload = unknown, Result = any> = {
    message: CustomEvent<SignedMessage<Payload>>;
    commit: CustomEvent<{
        root: Node;
        heads: string[];
    }>;
    sync: CustomEvent<{
        duration: number;
        messageCount: number;
        peer?: string;
    }>;
    connect: CustomEvent<{
        peer: string;
    }>;
    disconnect: CustomEvent<{
        peer: string;
    }>;
};
export type MessageRecord<Payload> = {
    id: string;
    signature: Signature;
    message: Message<Payload>;
    hash: string;
    branch: number;
    clock: number;
};
export declare abstract class AbstractGossipLog<Payload = unknown, Result = any> extends TypedEventEmitter<GossipLogEvents<Payload, Result>> {
    #private;
    static schema: {
        $branch_merges: {
            id: "primary";
            source_branch: "integer";
            source_message_id: "string";
            source_clock: "integer";
            target_branch: "integer";
            target_message_id: "string";
            target_clock: "integer";
        };
        $ancestors: {
            id: "primary";
            links: "json";
        };
        $messages: {
            id: "primary";
            signature: "json";
            message: "json";
            hash: "string";
            branch: "integer";
            clock: "integer";
            $indexes: string[];
        };
        $heads: {
            id: "primary";
        };
    };
    readonly topic: string;
    readonly signer: Signer<Payload>;
    readonly controller: AbortController;
    abstract db: AbstractModelDB;
    abstract tree: Tree;
    protected readonly log: Logger;
    readonly validatePayload: (payload: unknown) => payload is Payload;
    readonly verifySignature: (signature: Signature, message: Message<Payload>) => Awaitable<void>;
    protected constructor(init: GossipLogInit<Payload, Result>);
    close(): Promise<void>;
    replay(): Promise<void>;
    connect(url: string, options?: {
        signal?: AbortSignal;
    }): Promise<void>;
    listen(port: number, options?: {
        signal?: AbortSignal;
    }): Promise<void>;
    startLibp2p(config: NetworkConfig): Promise<Libp2p<ServiceMap<Payload>>>;
    encode<T extends Payload = Payload>(signature: Signature, message: Message<T>, context?: {
        source?: MessageSource;
        branch?: number;
    }): SignedMessage<T, Result>;
    decode(value: Uint8Array, context?: {
        source?: MessageSource;
        branch?: number;
    }): SignedMessage<Payload, Result>;
    getClock(): Promise<[clock: number, heads: string[]]>;
    has(id: string): Promise<boolean>;
    get(id: string): Promise<SignedMessage<Payload, Result> | null>;
    getMessages(range?: {
        lt?: string;
        lte?: string;
        gt?: string;
        gte?: string;
        reverse?: boolean;
        limit?: number;
    }): Promise<{
        id: string;
        signature: Signature;
        message: Message<Payload>;
        branch: number;
    }[]>;
    iterate(range?: {
        lt?: string;
        lte?: string;
        gt?: string;
        gte?: string;
        reverse?: boolean;
        limit?: number;
    }): AsyncIterable<SignedMessage<Payload, Result>>;
    /**
     * Sign and append a new *unsigned* message to the end of the log.
     * The current concurrent heads of the local log are used as parents.
     */
    append<T extends Payload = Payload>(payload: T, { signer }?: {
        signer?: Signer<Payload>;
    }): Promise<SignedMessage<T, Result> & {
        result: Result;
    }>;
    /**
     * Insert an existing signed message into the log (ie received via HTTP API).
     * If any of the parents are not present, throw an error.
     */
    insert(signedMessage: SignedMessage<Payload, Result>): Promise<{
        id: string;
    }>;
    private apply;
    private newBranch;
    private getBranch;
    isAncestor(id: string, ancestor: string, visited?: Set<string>): Promise<boolean>;
    /**
     * Sync with a remote source, applying and inserting all missing messages into the local log
     */
    sync(snapshot: SyncSnapshot, options?: {
        peer?: string;
    }): Promise<{
        complete: boolean;
        messageCount: number;
    }>;
    serve<T>(callback: (snapshot: SyncSnapshot) => Awaitable<T>): Promise<T>;
}
